#include "header.h"

#define M_PI	3.141592

#define NUMARCS	7
int arcSampleRate = 40;
typedef float PT2[2];
PT2 *arcPt;

typedef struct _Arc
{
	float startAngle;	// 0 ~ 1
	float endAngle;		// 0 ~ 1
	float radius;
	float center[2];
} _Arc;

_Arc _arc[NUMARCS];
static double colors[NUMARCS][3] =
{
	{ 0.2, 0.2, 0.2 }, 
	{ 0.0, 0.0, 1.0 }, 
	{ 0.0, 1.0, 0.0 }, 
	{ 0.0, 1.0, 1.0 }, 
	{ 1.0, 0.0, 0.0 }, 
	{ 1.0, 0.0, 1.0 }, 
	{ 1.0, 1.0, 0.0 }
};

void setArc(int idx, float s, float e, float r, float cx, float cy)
{
	_arc[idx].startAngle	= s;
	_arc[idx].endAngle		= e;
	_arc[idx].radius		= r;
	_arc[idx].center[0]		= cx;
	_arc[idx].center[1]		= cy;
}
void initArc()
{
	setArc(0, 0.55f, 0.8f, 0.3f, -0.4f, -0.4f);
	setArc(1, 0.2f, 0.5f, 0.25f, -0.4f, 0.f);
	setArc(2, -0.1f, 0.1f, 0.6f, -0.4f, -0.3f);
	setArc(3, 0.1f, 0.4f, 0.25f, 0.5f, -0.5f);
	setArc(4, 0.5f, 1.f, 0.25f, -0.1f, 0.5f);
	setArc(5, 0.f, 0.5f, 0.25f, 0.15f, 0.5f);
	setArc(6, 0.5f, 0.8f, 0.25f, 0.65f, 0.5f);
	arcPt = new PT2[arcSampleRate+1];
}
void freeArc()
{
	delete arcPt;
}
void drawArcDispMap(int idx)
{
	float sAngle = _arc[idx].startAngle;
	float eAngle = _arc[idx].endAngle;
	float cx = _arc[idx].center[0];
	float cy = _arc[idx].center[1];
	float radius = _arc[idx].radius;

	// evaluate arc point
	for (int i = 0; i <= arcSampleRate; i++)
	{
		float cAngle = (eAngle - sAngle)*(float)i/(float)arcSampleRate + sAngle;
		arcPt[i][0] = radius*cosf(cAngle*M_PI*2)+cx;
		arcPt[i][1] = radius*sinf(cAngle*M_PI*2)+cy;
	}

	const double kSize = 3000.0;	// MODIFIED !!!!

	// distance map
#ifdef _CONEDIFFCOLOR
	double color[3] = { colors[idx][0]-0.5, colors[idx][1]-0.5, colors[idx][2]-0.5 };
	if (color[0] < 0) color[0] = 0;
	if (color[1] < 0) color[1] = 0;
	if (color[2] < 0) color[2] = 0;
	glColor3dv(color);
#else
	glColor3dv(colors[idx]);
#endif
	glPushMatrix();
	glTranslated(radius*cosf(sAngle*M_PI*2)+cx, radius*sinf(sAngle*M_PI*2)+cy, -kSize); 
	glutSolidCone(kSize, kSize, 64, 1);
	glPopMatrix();
#ifdef _CONEDIFFCOLOR
	color[0] += 0.25;
	color[1] += 0.25;
	color[2] += 0.25;
	glColor3dv(color);
#endif
	glPushMatrix();	
	glTranslated(radius*cosf(eAngle*M_PI*2)+cx, radius*sinf(eAngle*M_PI*2)+cy, -kSize); 
	glutSolidCone(kSize, kSize, 64, 1);
	glPopMatrix();	

	glColor3dv(colors[idx]);
	for (int i = 0; i < arcSampleRate; i++)
	{
		// inner
		glBegin(GL_TRIANGLES);
		glVertex3f(arcPt[i][0], arcPt[i][1], 0);
		glVertex3f(cx, cy, -radius);
		glVertex3f(arcPt[i+1][0], arcPt[i+1][1], 0);
		glEnd();
		// outer
		glBegin(GL_QUADS); 
		glVertex3f(arcPt[i][0], arcPt[i][1], 0);
		glVertex3f((arcPt[i][0]-cx)/radius*kSize, (arcPt[i][1]-cy)/radius*kSize, -kSize);
		glVertex3f((arcPt[i+1][0]-cx)/radius*kSize, (arcPt[i+1][1]-cy)/radius*kSize, -kSize);
		glVertex3f(arcPt[i+1][0], arcPt[i+1][1], 0);
		glEnd();
	}

	// draw arc
	glColor3d(0.7, 0.7, 0.7);
	glLineWidth(1.5f);
	glBegin(GL_LINE_STRIP);
	for (int i = 0; i <= arcSampleRate; i++)
		glVertex3f(arcPt[i][0], arcPt[i][1], 0.1);
	glEnd();
}
void drawArcDispMap()
{
	for (int i=0; i<NUMARCS; ++i) drawArcDispMap(i);
}